/*
 * LOKI_2620.C
 *
 * Code to try out SixSonar board in MicroMoose
 *
 * 12/17/12 alm
 */


#ifdef _PIC18
  #include  <pic18.h>
#else
  #include <pic.h>
#endif

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "Loki_2620.h"
#include "PIChead.h"
#include "LokiI2C.h"
#include "serial.h"
#include "delay.h"


/* IR Linearizing Function parms */
#define IRm 6787.0
#define IRb 3.0
#define IRk 4.0
#define ADVscale (5.0 / 1024.0)

/* select output modes wanted */
#define WANT_ASCII
#define WANT_BINARY

	/* function prototypes */

void interrupt SystemISR(void);
void main();
void init(void);
void AbnormalOps(void);
void ADstart(void);
void ReadAD(void);
void SonarData(void);
unsigned char IRrange(void);

extern void I2C_Tasks(void);
extern unsigned int SRF08_Range, SRF08_Light, SRF08_Ver, CMPS03_Heading;

volatile unsigned int 	SystemTick, SystemTick2;
volatile unsigned char	Hours, Minutes, Seconds;
volatile unsigned char	SecondsTick;

unsigned char LastSeconds;

volatile BITS1 Event1;
volatile BITS2 Event2;
volatile BITS3 Event3;

unsigned char ADchan = 0;	/* change to only 4 A/D channels read */
unsigned int ADbuff[4];
unsigned int ADbuff1st[4];

float IRrangeLeft = 99.0, IRrangeRight = 99.0, IRrangeFloor = 99.0;
float IRFleft, IRFright, IRFfloor, TempVoltage, PotVoltage;
unsigned char ServoIndex = 0;

const char const SixStr[] = "LokiSonar\r\n";
/*-------------------------------------------------------------*/

void main()

	{
	unsigned char len;
	char *CmdStr;

	init();			/* init hardware */
	I2C_Init();		/* initialize I2C */

	PutString(SixStr);		/* NEW signon BEFORE init?*/

//	test = PRELOAD_TMR0;			/* preload timer 0 */
//	sprintf(Cmd, "TMR0 %d, INT_PERIOD %d, SCALE %d\r\n", test, INT_PERIOD, SCALE);	/* mode */
//	PutString(Cmd);

   /* Wait for commands: */

	do			/* FULL SPEED processor loop */
		{
		if(CHK_RX_MSG_FLAG)
			{
			if(CHK_RX_MSG_FLAG)
				CmdStr = GetString();	/* command stings from com port */

			if(CmdStr != NULL)
				len = CompressStr(&CmdStr);	/* remove white space and invalid chars from string */

			ParseCmds(CmdStr);			/* parse commands in command string */
			}

		I2C_Tasks();					/* I2C dispatch routine */

		SonarData();					/* fast reply to request for sonar data */

		if(CHK_SYSTEM_TICK)				/* called every 0.1 seconds */
			{
			AbnormalOps();				/* process reports */
			CLR_SYSTEM_TICK;			/* clear event flag */
			}
	 	}
	 while(1); // forever
	}


/*------------------------------------------------------------------------*/
void SonarData(void)

/*
 * Report the range
 *
 * Fast reply, not in AbnormalOps
 */

	{
	char Cmd[40];
	unsigned char len;

	if(CHK_RANGE_RPT)
		{
		srf08_addr += 0x02;		/* work through addresses of SRF08 module addresses */

		if(++srf08_chan > 5)	/* once we have collected 6 readings */
			{
#ifdef WANT_ASCII
			sprintf(Cmd, "S %04X %04X %04X %04X %04X %04X\r\n",
				srf08_data[0], srf08_data[1], srf08_data[2], srf08_data[3], srf08_data[4], srf08_data[5]);

			PutString(Cmd);
#endif // WANT_ASCII

#ifdef WANT_BINARY
			len = sprintf(Cmd, "S%c%c%c%c%c%c%c%c%c%c%c%c",
				srf08_data[0] & 0xFF, srf08_data[0]	>>8,
				srf08_data[1] & 0xFF, srf08_data[1]	>>8,
				srf08_data[2] & 0xFF, srf08_data[2] >>8,
				srf08_data[3] & 0xFF, srf08_data[3]	>>8,
				srf08_data[4] & 0xFF, srf08_data[4]	>>8,
				srf08_data[5] & 0xFF, srf08_data[5]	>>8	);

			PutData(Cmd, len);
#endif // WANT_BINARY

			srf08_addr = SRF08_000;	/* back to first */
			srf08_chan = 0;
			}

		CLR_RANGE_RPT;
		}
	}

/*------------------------------------------------------------------------*/
void AbnormalOps(void)

/*
 * called every .1 sec
 * handles reports, heartbeat
 *
 * uses "time slots" to spread out the sending of messages
 * 06/14/08 alm
 */

	{
	static unsigned char mux = 0;
	unsigned char i, len;
	signed int IR_L, IR_R;
	char Cmd[40];

	/* rotate through messages to send */

	switch(mux)	/* each task is checked once each second (spreads out the report tasks) */
		{
		case 0:
			{
//			if(CHK_RANGE_RPT & 0)
//				{
//				if((signed int)SRF08_Range > 0)
//					{
//					sprintf(Cmd, "Sonar %04X chan %0X\r\n", SRF08_Range, srf08_addr);
//					PutString(Cmd);
//					}
//
//				srf08_addr += 0x02;
//
//				if(++srf08_chan > 5)
//					{
//					sprintf(Cmd, "Sonar %04X %04X %04X %04X %04X %04X\r\n",
//						srf08_data[0], srf08_data[1], srf08_data[2], srf08_data[3], srf08_data[4], srf08_data[5]);
//
//					PutString(Cmd);
//
//					srf08_addr = SRF08_000;
//					srf08_chan = 0;
//					}
//
//				CLR_RANGE_RPT;
//				}
			}
		break;

		case 1:
			{
			if(CHK_LIGHT_RPT)
				{
#ifdef WANT_ASCII
//				sprintf(Cmd, "Light %d, vers. %d\r\n", SRF08_Light, SRF08_Ver);
				sprintf(Cmd, "L %d %d %d %d %d %d\r\n",
					srf08_LightData[0], srf08_LightData[1], srf08_LightData[2],
					srf08_LightData[3], srf08_LightData[4], srf08_LightData[5]);

				PutString(Cmd);
#endif // WANT_ASCII

#ifdef WANT_BINARY
				len = sprintf(Cmd, "L%c%c%c%c%c%c%c%c%c%c%c%c",
					srf08_LightData[0] & 0xFF, srf08_LightData[0] >>8,
					srf08_LightData[1] & 0xFF, srf08_LightData[1] >>8,
					srf08_LightData[2] & 0xFF, srf08_LightData[2] >>8,
					srf08_LightData[3] & 0xFF, srf08_LightData[3] >>8,
					srf08_LightData[4] & 0xFF, srf08_LightData[4] >>8,
					srf08_LightData[5] & 0xFF, srf08_LightData[5] >>8 );

				PutData(Cmd, len);
#endif // WANT_BINARY

				CLR_LIGHT_RPT;
				}
			}
		break;

		case 2:
			{
			if(CHK_COMPASS_RPT)
				{
//				sprintf(Cmd, "Head %03.1f, vers %d\r\n", (float)CMPS03_Heading/10.0, CMPS03_Ver);

				if(CMPS03_Heading <= 3600)
					{
#ifdef WANT_ASCII
//					sprintf(Cmd, "Heading %03.1f\r\n", (float)CMPS03_Heading/10.0);
					sprintf(Cmd, "H %03.1f\r\n", (float)CMPS03_Heading/10.0);

					PutString(Cmd);
#endif // WANT_ASCII

#ifdef WANT_BINARY
					len = sprintf(Cmd, "H%c%c",	CMPS03_Heading & 0xFF, CMPS03_Heading >> 8);

					PutData(Cmd, len);
#endif // WANT_BINARY
					}

				CLR_COMPASS_RPT;
				}
			}
		break;

		case 3:
			{
			if(CHK_AD_RPT)
				{
				if(CHK_AD_READY)	/* shared with above.  Two flags? */
					{
#ifdef WANT_ASCII
					for(len = 0, i = 0; i < 4; i++)		/* change to 4 channels */
						len += sprintf(&Cmd[len], "D ch%d %04X ", i, ADbuff[i]);

					len += sprintf(&Cmd[len], "\r\n");
					PutString(Cmd);
#endif // WANT_ASCII

#ifdef WANT_BINARY
					len = sprintf(Cmd, "D%c%c%c%c%c%c%c%c",
						ADbuff[0] & 0xFF, ADbuff[0] >> 8,
						ADbuff[1] & 0xFF, ADbuff[1] >> 8,
						ADbuff[2] & 0xFF, ADbuff[2] >> 8,
						ADbuff[3] & 0xFF, ADbuff[3] >> 8);

				PutData(Cmd, len);
#endif // WANT_BINARY

					CLR_AD_READY;	/* here?  shared with above */
					}

				CLR_AD_RPT;
				}
			}
		break;

		case 4:
			{
			}
		break;

		case 5:
			{
#ifdef WANT_ASCII
//			sprintf(Cmd, "TIME %02d:%02d:%02d\r\n", Hours, Minutes, Seconds);
//			PutString(Cmd);
#endif // WANT_ASCII
			}
		break;

		case 6:	/* currently unused states (take min 4 byes each!) */
			{
			}
		break;

		case 7:
			{
			if(CHK_IR_RPT)
				{
#ifdef WANT_ASCII
//	sprintf(Cmd, "L %6.1f, R %6.1f, F %6.1f\r\n", IRrangeLeft, IRrangeRight, IRrangeFloor);
				sprintf(Cmd, "IR L %6.1f\", R %6.1f\"\r\n", IRrangeLeft, IRrangeRight);
				PutString(Cmd);
#endif WANT_ASCII

#ifdef WANT_BINARY
				IR_L = (signed int)IRrangeLeft;
				IR_R = (signed int)IRrangeRight;

				len = sprintf(Cmd, "I%c%c%c%c",
				   IR_L & 0xFF, IR_L >> 8,
				   IR_R & 0xFF, IR_R >> 8 );

				PutData(Cmd, len);

#endif // WANT_BINARY

				CLR_IR_RPT;
				}
			}
		break;

		case 8:
			{
			if(CHK_TPA81_RPT)
				{
#ifdef WANT_ASCII
				sprintf(Cmd, "T %03d %03d %03d %03d %03d %03d %03d %03d %03d  %03d\r\n",
				tpa81_data[0], tpa81_data[1], tpa81_data[2], tpa81_data[3], tpa81_data[4],
				tpa81_data[5], tpa81_data[6], tpa81_data[7], tpa81_data[8], tpa81_data[9]  );

				PutString(Cmd);
#endif // WANT_ASCII

#ifdef WANT_BINARY
			len = sprintf(Cmd, "T%c%c%c%c%c%c%c%c%c%c",
				tpa81_data[0], tpa81_data[1], tpa81_data[2], tpa81_data[3], tpa81_data[4],
				tpa81_data[5], tpa81_data[6], tpa81_data[7], tpa81_data[8], tpa81_data[9]  );

			PutData(Cmd, len);
#endif // WANT_BINARY

				CLR_TPA81_RPT;
				}
			}
		break;

		case 9:			/* 10 states, each is tested every 1 sec. (0.1 sec * 10) */
			{
			}
		break;

		default:
			{
			mux = 0;
			}
		break;

		}// switch

	if(++mux > 9)	/* next task */
		mux = 0;


	/* 1 SECOND TIMING */

	if(++SecondsTick == SECONDS_TICK_DIVISOR)
		{
		SecondsTick = 0;		/* here once a second to count seconds */
		LED1_ON;			/* keep alive LED *ON* ONCE a SECOND (for 1/10 sec) */

		if(++Seconds == SECONDS_PER_MINUTE)
			{
			Seconds = 0;

			if(++Minutes == MINUTES_PER_HOUR)
				{
				Minutes = 0;

				if(++Hours == HOURS_PER_DAY)
					Hours = 0;
				}
			}// seconds tick rollover
		}
	else
		{
		LED1_OFF;			/* keep alive LED *OFF* */
		}
	}

/*------------------------------------------------------------------------*/
void ADstart(void)

/*
 * Select the A/D channel and start the conversion
 * 2/20/08 alm
 */

	{

	CHS0 = ( ADchan & 0x01);
	CHS1 = ((ADchan & 0x02) >> 1);
	CHS2 = ((ADchan & 0x04) >> 2);

	DelayMicrosec(10);

	GODONE = 1;		/* start A/D conversion */
	}


/*------------------------------------------------------------------------*/
void ReadAD(void)

/*
 * Interrupt routine to Read the ADC.
 * Approximate sampling time = 76.8us
 *
 * AN0/RA0 move left IR here
 * AN1/RA1 move right IR here
 * AN2/RA2
 * AN3/RA3

 * AN4/RA5
 * AN5/RE0 XXXleft IR
 * AN6/RE1 XXXright IR
 * AN7/RE2
 * 2/20/08 alm (old)
 *
 *  from QFPV.ASM
 *; This subroutine reads the temperature sensor.  The value is calculated with:
 *; Displaying Temperature = (Value * 1000) / 2048.  Reason being for temperatures below
 *; 125F, the ADC output can be right-justified.  The output must be scaled appropriately.
 *			sprintf(Cmd, "Temp(0) %5.1fF, pot(4) %03d\r\n", (ADbuff[0] * 1000.0) / 2048.0, ADbuff[4]);
 *			PutString(Cmd);
 *
 *  Same results as mine, ADVscale = (5.0 / 1024.0),   100 mV / F
 *		sprintf(Cmd, "Temp(0) %5.1fF, pot(4) %5.2fV\r\n", TempVoltage * 100.0, PotVoltage);
 */


/* old pin assignments */
//		TempVoltage = (float)ADbuff[0] * ADVscale;	/* calculate voltage from temp sensor */
//		PotVoltage  = (float)ADbuff[4] * ADVscale;	/* calculate voltage from pot */
//		IRFleft  	= (float)ADbuff[5] * ADVscale;	/* calculate left IR voltage */
//		IRFright 	= (float)ADbuff[6] * ADVscale;	/* calculate right IR voltage */
//		IRFfloor 	= (float)ADbuff[7] * ADVscale;	/* calculate floor IR voltage */

/* new IR assignments */
//		IRFleft  	= (float)ADbuff[0] * ADVscale;	/* calculate left IR voltage */
//		IRFright 	= (float)ADbuff[1] * ADVscale;	/* calculate right IR voltage */

	{
	static unsigned char sample = 0;
	unsigned int ADCval;
	unsigned char i;

//RC1 = 1;
	ADCval	=   ADRESL;			/* Get LSB of A/D */
	ADCval	+= (ADRESH << 8);	/* Get MSB of A/D */

//	ADbuff1st[ADchan] += ADCval;
	ADbuff[ADchan] = ADCval;

	if(++ADchan > 3)	/* change from 8 A/D channels read to 4  12/22/10 alm */
		{
		ADchan = 0;


	if(0)
		if(sample++ == 10)	/* time to calculate average */
			{
			for(i = 0; i < 8; i++)
				{
				ADbuff[i] = (ADbuff1st[i] + 5) / 10;
				ADbuff1st[i] = 0;
				}

			sample = 0;
			}
		}

	ADIF = 0;					/* Reset the ADC interupt bit */
	SET_AD_READY;
//RC1 = 0;
	}

/*------------------------------------------------------------------------*/

unsigned char IRrange(void)

/* IR Linearizing Function
 *
 * Returns true with valid data and new
 *
 * R = (1 / (m * V + b)) - k
 *
 * For a controller using 10-bit A2D measurements (integers
 * ranging from 0 to 1023) successful interpolation yields the following formula
 * for a GP2D12 sensor:
 * R = (6787 / (V - 3)) - 4
 * Where m'=6787, b'=3 and k=4.  The user must check the input V to prevent
 * divide-by-zero, checking that V is larger than b'.  For this formula, V must be
 * greater than 3.  With 10-bit integers, voltage measurements from a GP2D12 are
 * typically above 80 when something is in range.  If nothing is in front of the
 * sensor, the readings can drop to 0.
 *
 * http://www.acroname.com/robotics/info/articles/irlinear/irlinear.html
 * 2/21/08 alm
 *
 * http://www.picbasic.co.uk/forum/showthread.php?t=7705
 *
 * 11/27/08 alm
 *
 * GP2D12 sensor:
 * Range:  10 to 80cm
 * Update frequency / period:  25Hz / 40ms
 * Direction of the measured distance:
 * Very directional, due to the IR LED
 * Max admissible angle on flat surface: > 40
 * Power supply voltage: 4.5 to 5.5V
 * Noise on the analog output: < 200mV
 * Mean consumption:  35mA
 * Peak consumption:  about 200mA
 *
 * Detection range: 10 cm to 80 cm (4" to 31")
 * new sample once every 38.3 +/-9.6 msec, so let's say once every 50 msec.
 * The beam angle is 5 degrees. So you can rotate about 100 degrees/second
 * and still be able to detect an opponent. Any faster and you'll miss everything.
 *
 * GP2D12 sensor: 2.45v @ 10cm	(4")
 *				  0.45v @ 80 cm	(31.5")
 * 0cm and 130cm.??
 *
 * GP2D12
 * R = (6787 / (V - 3)) - 4
 * #define IRm 6787.0
 * #define IRb 3.0
 * #define IRk 4.0
 * #define ADVscale (5.0 / 1024.0)
 *
 * GP2D120
 * 4-30 cm
 * R = (2914 / (V + 5)) - 1
 * #define IRm 2914.0
 * #define IRb -5.0
 * #define IRk 1.0
 *
 * Update frequency / period: 25Hz / 40ms
 * power requirement 35-200 mA
 *
 * add floor sensor IRFfloor (AD7)
 */

	{
	unsigned char retval;
	static unsigned int IRleftLast, IRrightLast, IRfloorLast;
//	char Cmd[50];

	if((ADbuff[0] != IRleftLast) || (ADbuff[1] != IRrightLast) || (ADbuff[2] != IRfloorLast))
		{
//		IRFleft = (float)ADbuff[0] * ADVscale;	/* calclate voltage */

		if(ADbuff[0] > IRb)	/* min value prevents divide by 0 */
			{
			IRrangeLeft  = (IRm / ((float)ADbuff[0] - IRb) ) - IRk;	/* calculate range */
			IRrangeLeft /= 2.54;	/* convert to inches */

//			if(IRrangeLeft < 4.0)	/* data gets flaky < 4" */
//				IRrangeLeft = 4.0;
			}


//		IRFright = (float)ADbuff[1] * ADVscale;	/* calculate voltage */

		if(ADbuff[1] > IRb)
			{
			IRrangeRight = (IRm / ((float)ADbuff[1] - IRb) ) - IRk;	/* calculate range */
			IRrangeRight /= 2.54;	/* convert to inches */

//			if(IRrangeRight < 4.0)	/* data gets flaky < 4" */
//				IRrangeRight = 4.0;
			}

//		IRFfloor = (float)ADbuff[2] * ADVscale;	/* calculate voltage */

		if(ADbuff[2] > IRb)
			{
			IRrangeFloor = (IRm / ((float)ADbuff[2] - IRb) ) - IRk;	/* calculate range */
			IRrangeFloor /= 2.54;	/* convert to inches */

//			if(IRrangeFloor < 4.0)	/* data gets flaky < 4" */
//				IRrangeFloor = 4.0;
			}

		IRleftLast  = ADbuff[0];	/* update last readings */
		IRrightLast = ADbuff[1];
		IRfloorLast = ADbuff[2];

//		if(IRrangeRight >= 4.0 || IRrangeLeft >= 4.0)
//			retval = 1;

		if(ADbuff[0] > 80 || ADbuff[1] > 80)
			retval = 1;

//		if(IRrangeLeft > 50 || IRrangeRight > 50)
			{
//			sprintf(Cmd, "ADbuff[0] %d, ADbuff[1] %d\r\n", ADbuff[0], ADbuff[1]);
//			PutString(Cmd);
//		retval = 0;
			}
		}
	else
		retval = 0;

	return(retval);
	}

/*-------------------------------------------------------------*/
void interrupt SystemISR(void)

/*
 * Arrive here from either a TMR0 or TMR1 interrupt.
 *
 */

	{
	static unsigned char SystemTick3 = 0;
	static unsigned char SystemTick4 = 0;

//	if(SSPIF && SSPIE)	/* if I2C SS interrupt */
//		{
//		SET_SS_COPY;
//		SSPIE = 0;
//		}

	if(TMR1IF && TMR1IE)	/* TMR1 service (reserved for servos) */
		{
		TMR1IF = 0;	// Clear the TMR1 Interrupt Flag:
		TMR1ON = 0;	// Turn TMR1 off:
		}

	/* FRAME TIME for any servo 2500 uS */

	if(TMR0IF && TMR0IE)				/* TMR0 mask enabled and an interrup */
    	{
//	RC2 = 1;			/* TEST '452 test point (shows frame width 2500 uS) */
		TMR0IF	= 0;	/* clear the interrupt flag on 18F4620, 18F2620, 18F452, 16F877 */
		TMR0	= (const)PRELOAD_TMR0;		/* preload timer 0 */

		TMR1IE = 1;	/* ENABLE TIMER 1 interrupt on overflow */
		TMR1ON = 1;

/*--------------------*/

		if(++SystemTick == SYSTEM_TICK_DIVISOR)
			{
			SystemTick = 0;
			SET_SYSTEM_TICK;		/* system tick at 1/10 sec.	(100 mS) */
			}

/*--------------------*/

		++SystemTick2;	/* timer for I2C delays ONLY */

/*--------------------*/

		/* Start up A/D, Sonar and Compass processes at 100 mS rate */
		/* Did this one work?  too slow for A/D?  */

		if(++SystemTick3 == 40)		/* 2.5 mS * 40 = 100 mS */
			{
			SystemTick3 = 0;
			}
		else
			if(SystemTick3 == 26)	/*Start I2C task 100 mS rate */
				{
	//			SET_RANGE_READ;		/* SONAR (I2C) AUTOMATIC request */
				}
			else
				if(SystemTick3 == 13)	/*Start I2C task 100 mS rate */
					{
		//			SET_COMPASS_READ;		/* COMPASS (I2C) AUTOMATIC request */
					}

/*--------------------*/
	/*Start A/D, process rate */

//		if(++SystemTick4 == MS10_TICK)		/* 10 mS rate */
		if(++SystemTick4 == MS25_TICK)		/* 25 mS rate */
//		if(++SystemTick4 == MS50_TICK)		/* 50 mS rate */
			{
			SystemTick4 = 0;

			ADstart();				/* get set of A/D data (I/R, Pot, Temp) */
			}

/*--------------------*/
#ifdef WANT_INTERLEAVED
	/*Start up System Task, (*A/D*), Sonar and Compass processes at 100 mS rate INTERLEAVED */

		if(++SystemTick == SYSTEM_TICK_DIVISOR)	/* 40 for Loki */
			{
			SystemTick = 0;
			SET_SYSTEM_TICK;		/* system tick at 1/10 sec.	(100 mS) */
			}
		else
			if(SystemTick == 30)		/* !SEEMS TOO SLOW! 2.5 mS * 10 = 100 mS */
				{
				ADstart();				/* get set of A/D data (I/R, Pot, Temp) */
				}
			else
				if(SystemTick == 20)
					{
					if(++SystemTick3 == 3)	/*Start I2C task 200 mS rate */
						{
						SystemTick2 = 0;
						SET_RANGE_READ;				/* SONAR (I2C) request */
						}							/* Sonar needs 150 mS delay for ping return */
					else
						if(SystemTick3 == 2)		/* Start I2C task 200 mS rate */
							SET_COMPASS_READ;		/* COMPASS (I2C) request */
						else
							if(SystemTick3 == 1)		/* Start I2C task 200 mS rate */
								SET_AD_RPT;	/* religated to the left-over A/D reads */

					}
#endif // WANT_INTERLEAVED
/*--------------------*/
		}// if T0IF


	#ifdef _PIC18
		if(ADIF && ADIE)			/* use A/D */
			ReadAD();

		if(RCIF && RCIE)
			RxCharDevice();				/* service comm receive */

		#ifdef TX_INTERRUPT_ENABLED
			if(TXIF && TXIE)
				TxCharDevice();			/* service comm xmit */
		#endif
	#endif

//	RC2 = 0;				/* TEST '452 test point */
	}


/*------------------------------------------------------------------------*/

void I2C_Init(void)

/*
 * initI2C
 *
 * errata: Clock Rate w/BRG is wrong.  Use 59H instead of 99 (63h) A3, A4 silicon
 *
 * 1. Configure the SCL and SDA pins as outputs by
 *    clearing their corresponding TRIS bits.
 * 2. Force SCL and SDA low by clearing the
 *    corresponding LAT bits.
 * 3. While keeping the LAT bits clear, configure
 *    SCL and SDA as inputs by setting their TRIS
 *    bits.
 *
 * Once this is done, use the SSPCON1 and
 * SSPCON2 registers to configure the proper I2C
 * mode as before.
 */

	{
	unsigned char Temp;

	SSPEN = 0;  		/* Disable I2C Mode initially */

	/* per errata */
	TRISC &= 0b11100111;	/* SCL an SDA low */
	LATC  &= 0b11100111;
	TRISC |= 0b00011000;	/* SCL an SDA set as inputs */

	SSPSTAT = 0x80;		/* Slew rate is disabled */
//	SSPCON1 = 0x38;		/* Master mode, clock = FOSC/(4 * (SSPADD + 1)) Enable  */
	SSPCON1 = 0x18;		/* Master mode, clock = FOSC/(4 * (SSPADD + 1)) no enable yet */
	SSPCON2 = 0x00;

	/*	SSPAD = ((Fosc/BitRate)/4)-1 = (400/4)-1 = 99 */
//	SSPADD = 0x63;		/* =99 40MHz clock: Baud rate = 100kHz */
	SSPADD = 0x59;		/* =89 **A3 ERRATA**  40MHz clock: Baud rate = 100kHz */

	Temp = SSPBUF;		/* Dummy read clears SSPSTAT<BF> */

	SSPIF = 0;  		/* Clear Interrupt */
    BCLIF = 0;

	SSPEN = 1;  		/* enable I2C Mode (in SSPCON1) */
//	SSPIE = 1;			/* use an interrupt to set _SS_COPY flag */
	}

/*------------------------------------------------------------------------*/
void init(void)

	{
#if defined (_18F4620) || defined (_18F2620)	/* PICC18  */
  #define PIC_CLK 10000000

	#if defined(MPLAB_ICD)
		__CONFIG(1, HSPLL);
		__CONFIG(2, PWRTDIS & WDTDIS);
		__CONFIG(3, CCP2RB3 & PBANDIS);
		__CONFIG(4, LVPDIS & XINSTDIS & DEBUGEN);
	#else
		#ifdef PLL_X4
				__CONFIG(1, HSPLL);		/* OSCSEN ?  HSPLL */
		#else
				__CONFIG(1, HS);		/* OSCSEN ?  HSPLL */
		#endif
		__CONFIG(2, PWRTEN & BOREN & BORV46 & WDTDIS);	/* added brownout BORV46 */
		__CONFIG(3, CCP2RC1 & PBANDIS);
		__CONFIG(4, LVPDIS & XINSTDIS & DEBUGDIS);
		__CONFIG(5, UNPROTECT);
		__CONFIG(6, WRTEN);
	#endif
#endif // _18F4620

	/* clear the ports */
	PORTA = 0;
	PORTB = 0;

#ifdef __40_PIN
	PORTC = 0;
	PORTD = 0;
	PORTE = 0;
#endif

#ifdef QUICK_FLASH_BOARD /* '452 '4620 */

	TRISA = 0B00100001;	/* RA0 temp, RA1-RA4 LED, RA5 pot  */
	TRISB = 0B00110000;	/* SPI_DAT, SPI_CMD, SPI_SEL, SPI_CLK,  sw4 & sw5 in, 4 Servos  */
//	TRISC = 0B10010001;	/* CCP1 CCP2 (DAC_CS) */
	TRISC = 0B10111001;	/* RX,TX,SDO,SDI,SCK,CCP1,CCP2,ATN_CS  disconnect DAC!   SDA SCK for I2C hi for slave ??  */

	#ifdef __40_PIN
		TRISD = 0B00001111;	/* RD4-7 LCD out, RD0,2 encoder, RD2,3 SW3 */
	//	TRISE = 0B00000000;	/* RS, E, output lines, ENC2,  WATCH OUT FOR THE SLAVE! */
		TRISE = 0B00000111;	/* RE0/AN5, RE1/AN6, RE2/AN7 */
	#endif
#endif  // QUICK_FLASH_BOARD


#ifdef SCHMART_BOARD /* 18F2620 */
	TRISA = 0B11101111;	/* RA0-3 A/D, RA4 is Alive, */
	TRISB = 0B11111111;	/* nothing determined yet  */
	TRISC = 0B10100111; /* RC3 SCL, RC4 SDA   RC6 TX RC7 RX (more TRISC for I2C in I2C init) */
#endif // SCHMART_BOARD


/*-----------------------------------*/
    /* Initialize TRM0: */

#if defined (_18F4620) || defined (_18F452) || defined (_18F2620)

    /* Initialize TRM0 (in T0CON) */
	TMR0ON = 1;		/* Timer0 1 = on */
	T08BIT = 1;		/* Timer0 8 bit  = 1 */
	T0CS   = 0;		/* Timer0 0= internal clock, 1= T0CKI pin */
	T0SE   = 0;		/* Timer0 1=inc on H to L transision, 0= L to H */
	PSA    = 0;		/* Timer0 prescaler asigned	*/

	#ifdef SCALE_256
		T0PS2  = 1;	/* 3 bits for prescale, 111 = 256 prescale */
		T0PS1  = 1;
		T0PS0  = 1;
	#endif

	#ifdef SCALE_128
		T0PS2  = 1;	/* 3 bits for prescale, 110 = 128 prescale */
		T0PS1  = 1;
		T0PS0  = 0;
	#endif

	#ifdef SCALE_64	/* SERVO 8 */
		T0PS2  = 1;	/* 3 bits for prescale, 101 = 64 prescale */
		T0PS1  = 0;
		T0PS0  = 1;
	#endif

	#ifdef SCALE_16
		T0PS2  = 0;	/* 3 bits for prescale, 011 = 16 prescale */
		T0PS1  = 1;
		T0PS0  = 1;
	#endif

	#ifdef SCALE_8
		T0PS2  = 0;	/* 3 bits for prescale, 010 = 8 prescale */
		T0PS1  = 1;
		T0PS0  = 0;
	#endif

    /* Turn on TMR0 */
	TMR0IF	= 0;	/* clear timer1 interrupt flag */

	TMR0	= (const)tmr0_value_negative;
//	TMR0	= (const)((0 - 157) & 0xFF);	/* test */

	TMR0	= (const)PRELOAD_TMR0;		/* preload timer 0 */
	TMR0IE	= 1;	/* ENABLE TMR0 interrupt on overflow  */


	/* Initialize TIMER1.  Want int on overflow, peripherial int on (later) */

    T1CON	= 0;
//  T1RD16  = 1;	/* 16 bit operations			*/
	T1RD16	= 0;	/* (8 bit operations)			*/

//	T1CKPS0	= 1;	/* (prescale = 1:8)				*/
//	T1CKPS1	= 1;
	T1CKPS0	= 0;	/* (prescale = 1:1)				*/
	T1CKPS1	= 0;

//	T1OSCEN	= 0;	/* (lp osc. off)				*/
//	T1SYNC	= 0;	/* (ignored)					*/
	TMR1CS	= 0;	/* (Fosc/4)						*/
//	TMR1ON	= 0;	/* (off, int routine turns on)	*/

    TMR1IF	= 0;	/* clear timer1 interrupt flag	*/
    TMR1IE	= 0;	/* disable timer1 interrupt		*/
#endif // 18F4620 18F452


/*-----------------------------------*/

/*
 * 	SSP and serial I/O
 *  SPBRG = 64  == 9600 BAUD with 0.16% error. BRGH = 0
 *	SPBRG = 129 == 19.2K BRGH = 0
 *  1/26/06 alm
 *
 * select SERIAL_XTAL and BRGH_1 in serial.h
 *
 */

#define B115K   (25000/1156)


//	InitSerial(B9600);	/* SPBERG  9600 baud, BRGH_1 */
//	InitSerial(B19200);	/* SPBERG  19,200 baud, BRGH_1 */
	InitSerial(B115K);	/* SPBERG  115,000 baud, BRGH_1 */
	SET_ECHO_FLAG;		/* enable echo to terminal */

/*-----------------------------------*/

#if defined (_18F4620) || defined (_18F452) || defined (_18F2620)
/* setup A/D on Schmart or demo board */

	/* interrupt setup for A/D */
	ADIE	=	0;		 	 		/* Mask the interrupt (we'll check in INT) */
	ADIF 	=	0;					/* Reset the ADC interupt bit */

	ADRESL	=	0;					/* Reset the ADRES value register */
	ADRESH	=	0;

	ADCON1 = 0xCE;	/* Enable PORTA one ANALOG &  7 other digital I/O pins ADFM ADCS2 mine */
//	ADCON1 = 0xC7;	/* Enable PORTA ALL ANALOG and PORTE ALL ANALOG */

/* one analog channel turns on analog circuitry.  other 7 digital can ALSO be used analog */
/* Fosc/16 for analog circuitry 10 MHz */

	ADCON2 = 0xA1;	/* 61h  fosc/64 ADCS2=1, ADCS1=1, ADCS0=0 */
//	ADCON2 = 0x61;	/* 61h  fosc/8  ADCS2=0, ADCS1=0, ADCS0=1 */

	ADIP = 0;	/* A/D low priority */
	ADIE = 1;	/* enable A/D interrupt */

	CMCON = 7;	/* disable digital comparitors */
	ADON = 1;	/* A/D on, chan RA0/AD0 (in ADCON0) */

#endif /* A/D init on  4620 452 2620 */

/* Initializations */

	LED1_OFF;  /* 1 = off */
	LED2_OFF;  /* 1 = off */

	SecondsTick	= 0;
	Seconds		= 0;
	Minutes		= 0;
	Hours		= 0;

	Event1.Event1 = 0;	/* clear all event flags Y */
	Event2.Event2 = 0;
	Event3.Event3 = 0;

	TMR0IE	= 1;	/* TMR0 interrupt on overflow  */
	PEIE	= 1;	/* enable peripherial interrupts */
    GIE		= 1;	/* enable global interrupts */

	/* priority interupts ?? */
//	IPEN    = 1;    /* Enable PRIORITY interrupt logic. */
//  GIEH    = 1;    /* Enable high priority interrupts. */
//  GIEL    = 1;    /* Enable low priority interrupts. */

	}
/*--------------------------------------------------------------------------*/
