
/*
 * header file for Loki 2620 Project
 *
 */

/* define FLAGS for tasks and processes */

typedef union
{
  struct
  {
  unsigned	 bRANGE_RPT		:1;
  unsigned	 bLIGHT_RPT		:1;
  unsigned	 bVERS_RPT		:1;
  unsigned	 bAD_RPT		:1;
  unsigned	 bCOMPASS_RPT	:1;
  unsigned	 bTPA81_RPT		:1;
  unsigned	 bIR_RPT		:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  } Bit1;

  volatile unsigned int Event1;
} BITS1;


/* this group for I2C tasks */
typedef union
{
  struct
  {
  unsigned	 bRANGE_READ	:1;
  unsigned	 bLIGHT_READ	:1;
  unsigned	 bCOMPASS_READ	:1;
  unsigned	 bRANGE_SETUP	:1;
  unsigned	 bGAIN_SETUP	:1;
  unsigned	 bCOMPASS_CALIB	:1;
  unsigned	 bSS_COPY		:1;
  unsigned	 bNEW_ADDRESS	:1;
  unsigned	 bTPA81_READ	:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  unsigned	 				:1;
  } Bit2;

  volatile unsigned int Event2;
} BITS2;

/* more general use flags */
typedef union
{
  struct
  {
  unsigned   bSYSTEM_TICK	:1;
  unsigned   bRX_MSG_FLAG	:1;
  unsigned 	 bAD_READY		:1;
  unsigned   bECHO_FLAG		:1;
  unsigned	 				:1;
  unsigned 					:1;
  unsigned				   	:1;
  unsigned					:1;
  unsigned 					:1;
  unsigned					:1;
  unsigned					:1;
  unsigned					:1;
  unsigned   				:1;
  unsigned					:1;
  unsigned					:1;
  unsigned					:1;
  } Bit3;

  volatile unsigned int Event3;
} BITS3;


/* external data */
extern volatile BITS1 Event1;
extern volatile BITS2 Event2;
extern volatile BITS3 Event3;

/* external function prototypes */
extern void DelayMicrosec(unsigned char timer);
extern void InitSerial(unsigned char baud);
extern void RxCharDevice(void);
extern void TxCharDevice(void);
extern void PutString(const char *ptr);
extern void PutData(const char* ptr, unsigned char len);
extern void PutChar(char ch);
extern char *GetString(void);

/* parser functions */
extern unsigned int GetParm(char **str);
extern unsigned char CompressStr(char **str);
extern void ParseCmds(char *CmdStr);
extern char *GetCanned(void);
extern unsigned char CMPS03_Ver;
extern unsigned int ServoAD(unsigned char ServoNbr);
extern const char const SixStr[];
extern unsigned ParseBlock(char **cmd, unsigned int *Sequence1, unsigned int *Sequence2);
extern volatile unsigned int SystemTick, SystemTick2;

extern const char const LokiStr[];
extern unsigned char srf08_addr;
extern unsigned int  srf08_data[];
extern unsigned char tpa81_data[];
extern unsigned char srf08_LightData[];
extern unsigned char srf08_chan;
extern unsigned char NewSRF08_Address;
const unsigned char const SRF08_Chan[];

/* A/D conversion parameters */
extern unsigned int ADbuff[];
extern float IRrangeLeft, IRrangeRight, IRrangeFloor;

extern void I2C_Init(void);

/* defines */
/* Macros for timeouts */

/* (2500 uS) * 400 = 1 Sec */
#define MS1000_TICK 400

/* (2500 uS) * 200 = 0.5 Sec */
#define MS500_TICK 200

/* (2500 uS) * 60 = 150 mS */
#define MS150_TICK 60

/* (2500 uS) * 40 = 100 mS */
#define MS100_TICK 40

/* (2500 uS) * 20 = 50 mS */
#define MS50_TICK 20

/* (2500 uS) * 10 = 25 mS */
#define MS25_TICK 10

/* (2500 uS) * 8 = 20 mS */
#define MS20_TICK 8

/* (2500 uS) * 4 = 10 mS */
#define MS10_TICK 4

/* (2500 uS) * 2 = 5 mS */
#define MS5_TICK 2


#define LF	0x0A
#define FF	0x0C
#define CR	0x0D
#define ESC 0x1B

/* SRF08 defines */
#define SRF08_000	0xE0	/* default */
#define SRF08_060	0xE2
#define SRF08_120	0xE4
#define SRF08_180	0xE6
#define SRF08_240	0xE8
#define SRF08_300	0xEA

/* MACROS for task flags */

#define SET_RANGE_RPT	(Event1.Bit1.bRANGE_RPT = 1)
#define CLR_RANGE_RPT	(Event1.Bit1.bRANGE_RPT = 0)
#define CHK_RANGE_RPT	(Event1.Bit1.bRANGE_RPT & 1)

#define SET_LIGHT_RPT	(Event1.Bit1.bLIGHT_RPT = 1)
#define CLR_LIGHT_RPT	(Event1.Bit1.bLIGHT_RPT = 0)
#define CHK_LIGHT_RPT	(Event1.Bit1.bLIGHT_RPT & 1)

#define SET_VERS_RPT	(Event1.Bit1.bVERS_RPT = 1)
#define CLR_VERS_RPT	(Event1.Bit1.bVERS_RPT = 0)
#define CHK_VERS_RPT	(Event1.Bit1.bVERS_RPT & 1)

#define SET_AD_RPT		(Event1.Bit1.bAD_RPT = 1)
#define CLR_AD_RPT		(Event1.Bit1.bAD_RPT = 0)
#define CHK_AD_RPT		(Event1.Bit1.bAD_RPT & 1)


#define SET_COMPASS_RPT	(Event1.Bit1.bCOMPASS_RPT = 1)
#define CLR_COMPASS_RPT	(Event1.Bit1.bCOMPASS_RPT = 0)
#define CHK_COMPASS_RPT	(Event1.Bit1.bCOMPASS_RPT & 1)

#define SET_IR_RPT		(Event1.Bit1.bIR_RPT = 1)
#define CLR_IR_RPT		(Event1.Bit1.bIR_RPT = 0)
#define CHK_IR_RPT		(Event1.Bit1.bIR_RPT & 1)

#define SET_TPA81_RPT	(Event1.Bit1.bTPA81_RPT = 1)
#define CLR_TPA81_RPT	(Event1.Bit1.bTPA81_RPT = 0)
#define CHK_TPA81_RPT	(Event1.Bit1.bTPA81_RPT & 1)

//---------

#define SET_RANGE_READ	(Event2.Bit2.bRANGE_READ = 1)
#define CLR_RANGE_READ	(Event2.Bit2.bRANGE_READ = 0)
#define CHK_RANGE_READ	(Event2.Bit2.bRANGE_READ & 1)

#define SET_LIGHT_READ	(Event2.Bit2.bLIGHT_READ = 1)
#define CLR_LIGHT_READ	(Event2.Bit2.bLIGHT_READ = 0)
#define CHK_LIGHT_READ	(Event2.Bit2.bLIGHT_READ & 1)

#define SET_COMPASS_READ (Event2.Bit2.bCOMPASS_READ = 1)
#define CLR_COMPASS_READ (Event2.Bit2.bCOMPASS_READ = 0)
#define CHK_COMPASS_READ (Event2.Bit2.bCOMPASS_READ & 1)

#define SET_RANGE_SETUP	(Event2.Bit2.bRANGE_SETUP = 1)
#define CLR_RANGE_SETUP	(Event2.Bit2.bRANGE_SETUP = 0)
#define CHK_RANGE_SETUP	(Event2.Bit2.bRANGE_SETUP & 1)

#define SET_GAIN_SETUP	(Event2.Bit2.bGAIN_SETUP = 1)
#define CLR_GAIN_SETUP	(Event2.Bit2.bGAIN_SETUP = 0)
#define CHK_GAIN_SETUP	(Event2.Bit2.bGAIN_SETUP & 1)

#define SET_COMPASS_CALIB	(Event2.Bit2.bCOMPASS_CALIB = 1)
#define CLR_COMPASS_CALIB	(Event2.Bit2.bCOMPASS_CALIB = 0)
#define CHK_COMPASS_CALIB	(Event2.Bit2.bCOMPASS_CALIB & 1)

#define SET_NEW_ADDRESS	(Event2.Bit2.bNEW_ADDRESS = 1)
#define CLR_NEW_ADDRESS	(Event2.Bit2.bNEW_ADDRESS = 0)
#define CHK_NEW_ADDRESS	(Event2.Bit2.bNEW_ADDRESS & 1)

#define SET_TPA81_READ	(Event2.Bit2.bTPA81_READ = 1)
#define CLR_TPA81_READ	(Event2.Bit2.bTPA81_READ = 0)
#define CHK_TPA81_READ	(Event2.Bit2.bTPA81_READ & 1)

//---------------------------------

#define SET_20MS_TICK	(Event3.Bit3.b2MS_TICK = 1)
#define CLR_20MS_TICK	(Event3.Bit3.b2MS_TICK = 0)
#define CHK_20MS_TICK	(Event3.Bit3.b2MS_TICK & 1)

#define SET_SYSTEM_TICK	(Event3.Bit3.bSYSTEM_TICK = 1)
#define CLR_SYSTEM_TICK	(Event3.Bit3.bSYSTEM_TICK = 0)
#define CHK_SYSTEM_TICK	(Event3.Bit3.bSYSTEM_TICK & 1)

#define SET_RX_MSG_FLAG	(Event3.Bit3.bRX_MSG_FLAG = 1)
#define CLR_RX_MSG_FLAG	(Event3.Bit3.bRX_MSG_FLAG = 0)
#define CHK_RX_MSG_FLAG	(Event3.Bit3.bRX_MSG_FLAG & 1)

#define SET_AD_READY	(Event3.Bit3.bAD_READY = 1)
#define CLR_AD_READY	(Event3.Bit3.bAD_READY = 0)
#define CHK_AD_READY	(Event3.Bit3.bAD_READY & 1)

#define SET_ECHO_FLAG	(Event3.Bit3.bECHO_FLAG	 = 1)
#define CLR_ECHO_FLAG	(Event3.Bit3.bECHO_FLAG	 = 0)
#define CHK_ECHO_FLAG	(Event3.Bit3.bECHO_FLAG	 & 1)

