/*
 * PIC header file for Loki2620.C
 * 12/2/10 alm
 */

/*
 * BOARD TYPES
 *
 * QUICK_FLASH_BOARD
 * SCHMART_BOARD
 */

/* choose BOARDS AUTOMATICALLY from MPLAB */

#if defined (_18F452) || defined (_18F4620)
	#define QUICK_FLASH_BOARD 1
#endif

#assert QUICK_FLASH_BOARD == 0

#if defined (_18F2620)
	#define SCHMART_BOARD 1
#endif

#assert SCHMART_BOARD == 1

/*-----------------------------------------------------------*/

	/* QUICK FLASH DEMO BOARD (MAJORITY BOARD) */

#ifdef QUICK_FLASH_BOARD

	#define TEMP1	RA0			/* A/D AN0 temp sensor 	  I	*/
	#define LED6	RA1			/* A/D Right LED?		  O */
	#define LED5	RA2			/* Vref Center LED?		  O	*/
	#define LED4	RA3			/* Vref Left LED?		  O	*/
	#define LED2	RA4			/* A/D Alive LED?		  O	*/
	#define POT1	RA5			/* A/D AN4 input pot	  I	*/

	#define servo0	RB0			/* pin 33				  O	*/
	#define servo1	RB1			/* pin 34				  O */
	#define servo2	RB2			/* pin 35				  O	*/
	#define servo3	RB3			/* pin 36				  O	*/
	#define servo4	RB4			/* pin 37				  O */
	#define servo5	RB5			/* pin 38				  O	*/
	#define servo6	RB6			/* pin 39				  O	*/
	#define servo7	RB7			/* pin 40				  O */

	#define	DAC_CS	RC0			/* DAC CS  TMR1 input???  I */
	#define	CCP2	RC1			/* CCP2 H3 header		  O */
	#define	TACH1	RC2			/* CCP1 int				  O */
	#define	SCLK	RC3			/* DAC SCLK				  O */
	#define SDI		RC4			/* SS  SDI	*SCL*		  O	*/
	#define SDO		RC5			/* DAC DIN	*SDA*		  O */
	#define TX		RC6			/* XMIT 				  O */
	#define RX		RC7			/* RECV					  I */

	#define LCD_DATA PORTD

	#define ENC1	RD0			/* Encoder A			  I */
	#define ENC2	RD1			/* Encoder B			  I */
	#define UNUSED4	RD2			/* 						  I */
	#define SW3		RD3			/* Switch 3				  I */
	#define LCD_D4	RD4			/* LCD data D4			  O */
	#define LCD_D5	RD5			/* LCD data D5			  O */
	#define LCD_D6	RD6			/* LCD data D6			  O */
	#define LCD_D7	RD7			/* LCD data D7			  O */

	#define AN5		RE0			/* A/D 5  Left IR         1	*/
	#define AN6		RE1			/* A/D 6  Right IR		  1 */
	#define AN7		RE2			/* A/D 7  				  1 */

//	#define LCD_RS	RE0			/* command/data           O	*/
//	#define LCD_EN	RE1			/* LCD clock  E			  O */
//	#define UNUSED5	RE2			/* 						  O */

	#define ENC1_BIT	0x01	/* sw input bits (do we care?) */
	#define ENC2_BIT	0x02
	#define UNUSED4_BIT	0x04
	#define SW3_BIT		0x08
	#define SDT_MASK (ENC1_BIT | ENC2_BIT | SW3_BIT)
	#define LCD_NIBBLE 0x0F0	/* high order nibble LCD	*/

	/*
	 * Switch1 is power switch
	 * Switch2 is RESET switch
	 */

//	#define SW_MASK	0x30		/* sw4 and sw5				*/

#endif // QUICK_FLASH_BOARD


/*-----------------------------------------------------------*/
#ifdef SCHMART_BOARD

								/* MCLR					   	pin 1  */

	#define AN0		RA0			/* A/D  			 	  I	pin 2  */
	#define AN1		RA1			/* A/D 				  	  I	pin 3  */
	#define AN2		RA2			/* A/D 					  I	pin 4  */
	#define AN3		RA3			/* A/D 					  I	pin 5  */
	#define LED1	RA4			/* LED	*** Alive LED	  O	pin 6  */
	#define LED2	RA5			/* LED   	  			  O	pin 7  */

								/* VSS					   	pin 8  */
								/* XTAL					    pin 9  */
								/* XTAL					   	pin 10 */

	#define	CKI		RC0			/* DAC CS  TMR1 input	  I pin 11 */
	#define	CCP2	RC1			/* CCP2 H3 header		  O pin 12 */
	#define	CCP1	RC2			/* CCP1 int				  O pin 13 */
	#define	SCL		RC3			/* I2C SCL				  O pin 14 */
	#define SDA		RC4			/* SS  SDA				  O	pin 15 */
	#define SDO		RC5			/* DAC DIN				  O pin 16 */
	#define TX		RC6			/* XMIT 				  O pin 17 */
	#define RX		RC7			/* RECV					  I pin 18 */

								/* VSS					   	pin 19 */
								/* VDD					   	pin 20 */

	#define AN12   	RB0			/* 				  		  I	pin 21 */
	#define AN10  	RB1			/* 				  		  I pin 22 */
	#define AN8   	RB2			/* 				  		  I	pin 23 */
	#define AN9   	RB3			/* 				  		  I	pin 24 */
	#define AN11  	RB4			/* 				  		  I pin 25 */

//	#define PGM   	RB5			/* PGM			  		  	pin 26 */
//	#define PGC   	RB6			/* PGC			  		  	pin 27 */
//	#define PGD   	RB7			/* PGD			  		    pin 28 */


#endif // SCHMART_BOARD

/*-----------------------------------------------------------*/


	/* defines for various pin levels */

/* LED control */
#define LED1_ON	 (LED1 = 0)
#define LED1_OFF (LED1 = 1)
#define LED2_ON	 (LED2 = 0)
#define LED2_OFF (LED2 = 1)


/*
 *
 * We want the TRM0 to interrupt every 2.5mS.  With FOSC=16MHz,
 * we need to interrupt every 2.5 * 1000 * 4 = 10,000 instructions.
 * With the TMR0 prescaler set to 64, we want TMR0 every 157 ticks
 * (157 * 64 = 10048):
 *
 * 01/09/08 alm
 */

#if defined (_18F452) || defined (_18F4620) || defined (_18F2620)		/* PICC18 */
	#define PLL_X4				 4	/* HS PLL for 18F parts 4X */

	#ifdef PLL_X4
		#define	XTAL	  40000000	/* crystal frequency - 40MHz  */
		#define SCALE_256			/* needed at 40MHz */
	#else
		#define	XTAL	  10000000	/* crystal frequency - 10MHz l */
		#define SCALE_64			/* for 10MHz */
	#endif
#else
	#define	XTAL  		  10000000	/* crystal frequency - 10MHz xtal Servo8 */
	#define SCALE_64				/* for 10MHz */
#endif // PICC18

#if defined (_16F877) || defined (_16F877A)
	#define	XTAL  		  10000000	/* crystal frequency - 10MHz w/xtal or Ceramic Resonator */
#endif


//#define TMR0_16				16	/* 16 bit TMR0 on 18F chips */
#define TMR0_8 					 8	/*  8 bit TMR0 on 16F chips */

#if defined QUICK_FLASH_BOARD || defined SCHMART_BOARD
	#ifdef PLL_X4	/* PLL ON */
		#define	INT_PERIOD		2500	/* 40Mhz, period of T0 interrupt in uS */
	#else								/* PLL OFF (seems we can't get step up as fast) */
		#define	INT_PERIOD		500		/* 10Mhz, period of T0 interrupt in uS */
	#endif
#else	/* 877 */
	#define	INT_PERIOD			500	/* 10Mhz, period of T0 interrupt in uS */
#endif

#assert INT_PERIOD == 2500 /* for 40Mhz */

#define SYSTEM_TICK_PERIOD	 100000	/* period of system tick in uS */
#define SPI_TICK_PERIOD		   1000	/* period of SPI in uS */
#define KHZ_PERIOD			   1000	/* period of 1KHz freq in uS */
#define SEC_PERIOD		    1000000	/* period of 1 second in uS */
#define MS_PERIOD			   1000	/* mS period in uS */
#define RECIPRICAL_uS		1000000	/* RECIPRICAL of 1 uS period */

#define	ICLK			   (XTAL/4)	/* crystal is divided by four */

/* select one of the following (above) to prescale TMR0 system clock */
//#define SCALE_256			/* needed at 40MHz */
//#define SCALE_128
//#define SCALE_64			/* not enough for 40MHz */
//#define SCALE_16
//#define SCALE_8			/* 4Mhz prescale by 16 - check for overflow! */

#ifdef SCALE_256
	#define	SCALE	256		/* prescale by 256 - check for overflow! */
#endif

#ifdef SCALE_128
	#define	SCALE	128		/* prescale by 128 - check for overflow! */
#endif

#ifdef SCALE_64
	#define	SCALE	64		/* prescale by 64 - check for overflow! */
#endif

#ifdef SCALE_16
	#define	SCALE	16		/* prescale by 16 - check for overflow! */
#endif

#ifdef SCALE_8
	#define	SCALE	8		/* prescale by 8 - check for overflow! */
#endif

/* time calculations (PERIOD) */


/* PRELOAD_TMR0  */
	#ifdef TMR0_16
		#define	PRELOAD_TMR0 (65536 - (INT_PERIOD * ICLK / SCALE / RECIPRICAL_uS) + 2)
		#define PRELOAD_TMR0L (PRELOAD_TMR0 & 0xFF)
		#define PRELOAD_TMR0H (PRELOAD_TMR0 >> 8)

		#assert PRELOAD_TMR0 < 65500	// 16 bit counter
	#else
		#if defined (_18F4620) || defined (_18F452) || defined (_18F2620)
			#define	PRELOAD_TMR0 (256 - ((INT_PERIOD * (ICLK / RECIPRICAL_uS) / SCALE))) /*    */
		#endif


	#ifdef PLL_X4
		#assert XTAL == 40000000		/* 40 mhz */
		#assert ICLK == 10000000		/* 10 mhz */
	#else
		#assert XTAL == 10000000		/* 10 mhz */
		#assert ICLK ==  2500000		/* 10 mhz */
	#endif
#endif /* TMR0 is 8 bit */

// Set up the clock
#define fosc XTAL
#define mips (fosc / 4)
#define microsecond (mips / RECIPRICAL_uS)
#define tmr0_instructions	(INT_PERIOD * microsecond)
#define tmr0_prescale SCALE
#define tmr0_value			(tmr0_instructions / tmr0_prescale + 1)
#define tmr0_value_negative	((0 - tmr0_value) & 0xFF)

#define SYSTEM_TICK_DIVISOR (SYSTEM_TICK_PERIOD / INT_PERIOD)
#assert SYSTEM_TICK_DIVISOR == 40

#define SECONDS_TICK_DIVISOR (SEC_PERIOD / SYSTEM_TICK_PERIOD)
#assert SECONDS_TICK_DIVISOR == 10


#define SECONDS_PER_MINUTE 60		/* constant data */
#define MINUTES_PER_HOUR 60
#define HOURS_PER_DAY 24


 /*------------------------------------------------------------------
 *
 * RJ11 to RJ11 ICD2 cable
 * PICbook demo board 18F452
 * USB demo board 18F4550
 * SCHMART board 18F2620
 * per MPLAB
 *
 * 1 MCLR
 * 2 VDD +5
 * 3 VSS GND
 * 4 RB7/PGD
 * 5 RB6/PGC
 * 6 N/C
 */

/*------------------------------------------------------------------
