/*
 *    Serial.h
 * Definitions and constants.
 *
 *  SPBRG    = 64;  9600 BAUD with 0.16% error. BRGH = 0
 * these calcs below are still funny, MicroChip table says above,
 * for 40MHZ fosc.
 *
 */


/* Enable BRGH since Xtal is high frequency */
#if defined (_18F452) || defined (_18F4620)	|| defined (_18F2620)	/* PICC18 */
	#define	SERIAL_XTAL	SXTAL40
#endif

#if defined (_18F4550) || defined (_18F2550)	/* PICC18 */
	#define	SERIAL_XTAL	SXTAL48		// USB! FOR '4550
#endif

#define BRGH_1 1

#define SXTAL10		 6250		// Use BRGH = 1
#define	SXTAL20		12500		// Use BRGH = 1
#define	SXTAL40		25000		// Use BRGH = 1
#define	SXTAL48		30000		// Use BRGH = 1


#ifdef BRGH_1
	/* Values for the SPBRG register with BRGH=1 */
	#define B9600	((SERIAL_XTAL/96)-1)
	#define B19200	((SERIAL_XTAL/192)-1)
	#define B56K	(SERIAL_XTAL/576)
	#define B115K   (SERIAL_XTAL/1156)

	/* Value for TXSTA */
	#define TXINIT		0x24	/* TXEN=1, BRGH=1 */
#else
	#define B9600	((SERIAL_XTAL/96)-1)
	#define B19200	((SERIAL_XTAL/192)-1)

	/* Value for TXSTA */
	#define TXINIT		0x20	/* TXEN=1, BRGH=0 */
#endif

#assert BRGH_1 == 1

#assert B115K == (25000/1156)


/* Value for RXSTA */
//#define RXINIT		0x80		/* SPEN=1, CREN=0, RX enabled */
#define RXINIT		0x90		/* SPEN=1, CREN=1, continuous, RX enabled */

/* BUFF_SIZE must be power of 2? */
#define RX_BUFF_SIZE  32
#define	RX_BUFF_MASK	RX_BUFF_SIZE-1

#define TX_BUFF_SIZE  32
#define	TX_BUFF_MASK	TX_BUFF_SIZE-1

extern char OutputBuffer[];

extern void InitSerial(unsigned char baud);
extern void TxCharDevice(void);
extern void RxCharDevice(void);
extern void PutChar(char ch);
extern unsigned char GetChar(void);
extern void PutString(const char *ptr);
extern void PutData(const char *ptr, unsigned char len);
extern char *GetString(void);
extern void PutCRLF(void);

/* more if needed */
//void PutBuffer(void);
//void PutInputLine(void);
//unsigned char getc(void);
//void ungetc(void);
//unsigned char GetHexNibl(void);
//unsigned char GetHexByte(void);
//void SetEchoFlag(unsigned char flg);
//void GetHexLong(void);
//unsigned int GetHexWord(void);
//unsigned int GetWord(void);
